"""Const exports"""

from jbqt.common import consts, qt_utils
from jbqt.common.consts import COLORS, STYLES, SIZES, GlobalRefs, ICONS
from jbqt.common.qt_utils import register_app

__all__ = [
    "COLORS",
    "consts",
    "GlobalRefs",
    "ICONS",
    "qt_utils",
    "register_app",
    "SIZES",
    "STYLES",
]
